#include "Model.h"

Model::Model()
{}

Model::~Model()
{}

bool Model::Init(GLuint sh, bool withText)
{
	projectionMatrix = glm::perspective(45.0f, (GLfloat)4.0f / (GLfloat)3.0f, 0.1f, 1000.0f);
	shader = sh;
	for (auto i = 0; i < objects.size(); i++)
	{
		objects[i]->CreateBuffers(withText);
		objects[i]->SetAttributeArrays(glGetAttribLocation(shader, "coord"), 0);
		objects[i]->SetAttributeArrays(glGetAttribLocation(shader, "color"), 1);
		objects[i]->SetAttributeArrays(glGetAttribLocation(shader, "texture"), 2);
		objects[i]->SetAttributeArrays(glGetAttribLocation(shader, "normal"), 3);
	}
	return true;
}

bool Model::Draw(GLuint Unif_mat, Camera* view)
{
	matrix = projectionMatrix * view->GetMatrix() * modelMatrix;
	glUniformMatrix4fv(Unif_mat, 1, GL_FALSE, &matrix[0][0]);
	GLuint p = glGetUniformLocation(shader, "normalMatrix");
	mat3 normM = transpose(mat3(inverse(modelMatrix)));
	glUniformMatrix3fv(p, 1, GL_FALSE, &normM[0][0]);
	for (auto i = 0; i < objects.size(); i++)
	{
		objects[i]->Draw(shader);
	}	
	return true;
}

bool Model::SetAttributeLocation(GLuint prog, const GLchar* name, int type)
{
	GLuint location = -1;
	for (auto i = 0; i < objects.size(); i++)
	{
		switch (type)
		{
		case 0: objects[i]->SetAttributeArrays(glGetAttribLocation(prog, name),0);
			break;
		case 1: objects[i]->SetAttributeArrays(glGetAttribLocation(prog, name), 1);
			break;
		case 2: objects[i]->SetAttributeArrays(glGetAttribLocation(prog, name), 2);
			break;
		case 3: objects[i]->SetAttributeArrays(glGetAttribLocation(prog, name), 3);
		}
	}
	return true;
}

bool Model::LoadModel()
{
	if (fileName == "")
		return false;
	const char* path = fileName.c_str();
	int len = strlen(path);
	char fileType[4];
	fileType[0] = path[len - 4];
	fileType[1] = path[len - 3];
	fileType[2] = path[len - 2];
	fileType[3] = path[len - 1];
	if (strcmp(strstr(path, ".obj"), ".obj") == 0){
		LoadModelFromObj(path);
		return true;
	}
	return false;
}

void Model::Translate(vec3 v)
{
	modelMatrix = glm::translate(modelMatrix, v);
	ms.position.x += v.x;
	ms.position.y += v.y;
	ms.position.z += v.z;
}

void Model::Scalling(vec3 v)
{
	modelMatrix = glm::scale(modelMatrix, v);
	ms.position.x *= v.x;
	ms.position.y *= v.y;
	ms.position.z *= v.z;
	ms.width *= v.x;
	ms.height *= v.y;
	ms.lenght *= v.z;
}

void Model::Rotate(vec3 v, float an){
	modelMatrix = rotate(modelMatrix, an, v);
}

void Model::SetProjection(float angle, float ratio, float first, float second){
	projectionMatrix = glm::perspective(angle, ratio, first, second);
}

bool Model::LoadModelFromObj(const char* file)
{
	FILE *f = fopen(file, "rb");
	if (!f)
		return false;
	string textLine;
	char buffer[100];
	int num_objects = 0;
	float maxX(0.0f), maxY(0.0f), maxZ(0.0f), minX(0.0f), minY(0.0f), minZ(0.0f);
	num_objects = -1;
	int state = 2;
	string mtlfile;
	while (!feof(f))
	{
		fgets(buffer, 100, f);
		textLine.clear();
		textLine += buffer;
		if (textLine.find("mtllib ") != std::string::npos && textLine.find("mtllib ") == 0){
			textLine = textLine.substr(7);
			textLine.pop_back();
			textLine.pop_back();
			mtlfile = textLine;
			continue;
		}
		else if (buffer[0] == 'v' && buffer[1]==' '){
			if (state == 2){
				num_objects++;
				state++;
				objects.push_back(new Object());
			}
			glm::vec3 _v = ParseVertex(buffer);
			objects[num_objects]->LoadVertexes(_v);
			if (_v.x >= maxX)
				maxX = _v.x;
			if (_v.x < minX)
				minX = _v.x;
			if (_v.y >= maxY)
				maxY = _v.y;
			if (_v.y < minY)
				minY = _v.y;
			if (_v.z >= maxZ)
				maxZ = _v.z;
			if (_v.z < minZ)
				minZ = _v.z;
		}
		else if (buffer[0] == 'v' && buffer[1] == 't'){
			if (state == 0)
				state++;
			objects[num_objects]->LoadTextures(ParseTexture(buffer));
		}
		else if (buffer[0] == 'v' && buffer[1] == 'n'){
			if (state == 1)
				state++;
			objects[num_objects]->LoadNormal(ParseNormal(buffer));
		}
		else if (buffer[0] == 'f' && buffer[1] == ' '){
			if (state == 2)
				state++;
			objects[num_objects]->LoadIndexes(ParseIndex(buffer, 0), 0);
			objects[num_objects]->LoadIndexes(ParseIndex(buffer, 1), 1);
			objects[num_objects]->LoadIndexes(ParseIndex(buffer, 2), 2);
		}
	}
	ms.maxP = vec3(maxX, maxY, maxZ);
	ms.minP = vec3(minX, minY, minZ);
	ParseMtlFile(mtlfile.c_str());
	ms.width = abs(maxX - minX);
	ms.height = abs(maxY - minY);
	ms.lenght = abs(maxZ - minZ);
	ms.position.x = 0.0f;
	ms.position.y = 0.0f;
	ms.position.z = 0.0f;
	fclose(f);
	return true;
}

glm::vec3 Model::ParseVertex(const char* buffer)
{
	string num;
	glm::vec3 v;
	int id = 0;
	for (auto i = 0; i < strlen(buffer); i++)
	{
		if ((buffer[i] >= 45 && buffer[i] <= 57) && buffer[i] != 47){
			num += buffer[i];
		}
		else
		{
			if (num.length() != 0){
				if (id == 0)
					v.x = atof(num.c_str());
				else if (id == 1)
					v.y = atof(num.c_str());
				else v.z = atof(num.c_str());
				num.clear();
				id++;
			}
		}
	}
	return v;
}

glm::vec2 Model::ParseTexture(const char* buffer)
{
	string num;
	glm::vec2 t;
	int id = 0;
	for (auto i = 0; i < strlen(buffer); i++)
	{
		if ((buffer[i] >= 45 && buffer[i] <= 57) && buffer[i] != 47){
			num += buffer[i];
		}
		else
		{
			if (num.length() != 0){
				if (id == 0)
					t.x = atof(num.c_str());
				else if (id == 1)
					t.y = atof(num.c_str());
				num.clear();
				id++;
			}
		}
	}
	return t;
}

glm::vec3 Model::ParseNormal(const char* buffer)
{
	string num;
	glm::vec3 n;
	int id = 0;
	for (auto i = 0; i < strlen(buffer); i++)
	{
		if ((buffer[i] >= 45 && buffer[i] <= 57) && buffer[i] != 47){
			num += buffer[i];
		}
		else
		{
			if (num.length() != 0){
				if (id == 0)
					n.x = atof(num.c_str());
				else if (id == 1)
					n.y = atof(num.c_str());
				else n.z = atof(num.c_str());
				num.clear();
				id++;
			}
		}
	}
	return n;
}

glm::vec3 Model::ParseIndex(const char* buffer, int type)
{
	string num;
	glm::vec3 ind;
	int id = 0;
	for (auto i = 0; i < strlen(buffer); i++)
	{
		if ((buffer[i] >= 45 && buffer[i] <= 57) && buffer[i] != 47){
			num += buffer[i];
		}
		else
		{
			if (num.length() != 0){
				if (type == 0){
					switch (id)
					{
					case 0: ind.x = atoi(num.c_str());
						break;
					case 3: ind.y = atoi(num.c_str());
						break;
					case 6: ind.z = atoi(num.c_str());
						break;
					default: break;
					}
				}
				else if (type == 1){
					switch (id)
					{
					case 1: ind.x = atoi(num.c_str());
						break;
					case 4: ind.y = atoi(num.c_str());
						break;
					case 7: ind.z = atoi(num.c_str());
						break;
					}
				}
				else if (type == 2){
					switch (id)
					{
					case 2: ind.x = atoi(num.c_str());
						break;
					case 5: ind.y = atoi(num.c_str());
						break;
					case 8: ind.z = atoi(num.c_str());
						break;
					}
				}
				num.clear();
				id++;
			}
		}
	}
	return ind;
}

void Model::ParseMtlFile(string path)
{
	FILE *f = fopen(("E:\\Development\\OpenGl\\Stage_D\\Stage_D\\Models\\" + path).c_str(), "rb");
	if (!f)
		return;
	char buffer[100];
	string str;
	int ind = -1;
	while (!feof(f))
	{
		fgets(buffer, 100, f);
		str.clear();
		str += buffer;
		if (str.find("newmtl ") != std::string::npos){
			ind++;
			continue;
		}
		if (str.find("map_") != std::string::npos){
			str = str.substr(str.find(" ") + 1);
			str.pop_back();
			str.pop_back();
			if (objects[ind]->textureFile != str){
				objects[ind]->textureFile = str.c_str();
				objects[ind]->useTexture = true;
			}
		}
	}
}

