#include "Object.h"

Object::Object()
{}

Object::Object(int vc, int tc, int ic)
{
	vertexCount = vc;
	textureCount = tc;
	indexCount = ic;
}

Object::~Object()
{}

void Object::CreateBuffers(bool withText)
{
	decIndexes();
	bool tb = withText;
	if (tb)
	{
		if (textureFile != ""){
			VertexTexture _vt;
			for (auto i = 0; i < vertexCount; i++)
			{
				_vt.vert = vertecies[i];
				for (auto j = 0; j < vIndex.size(); j++)
				{
					if (vIndex[j] == i){
						_vt.text = texture[tIndex[j]];
						_vt.norm = normals[nIndex[j]];
						break;
					}
				}
				data.push_back(_vt);
			}
			glGenBuffers(1, &vbo);
			glBindBuffer(GL_ARRAY_BUFFER, vbo);
			glBufferData(GL_ARRAY_BUFFER, data.size() * sizeof(VertexTexture), &data[0], GL_STATIC_DRAW);

			glGenBuffers(1, &ibo);
			glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, ibo);
			glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(unsigned int)*vIndex.size(), &vIndex[0], GL_STATIC_DRAW);
			LoadTextureFromFile();
		}
		else tb = false;
	}
	if (!tb){
		glGenBuffers(1, &vbo);
		glBindBuffer(GL_ARRAY_BUFFER, vbo);
		glBufferData(GL_ARRAY_BUFFER, vertecies.size() * sizeof(glm::vec3), &vertecies[0], GL_STATIC_DRAW);

		glGenBuffers(1, &ibo);
		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, ibo);
		glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(unsigned int)*vIndex.size(), &vIndex[0], GL_STATIC_DRAW);
	}
}

bool Object::LoadTextureFromFile()
{
	unsigned char header[54]; //  BMP-   ,   54 
	unsigned int dataPos;     //     ( )
	unsigned int width, height;
	unsigned int imageSize;   //   =  *  * 3
	// RGB-,   
	unsigned char * data;
	FILE * file = fopen(textureFile.c_str(), "rb");
	if (!file) {
		printf("    n");
		return 0;
	}
	if (fread(header, 1, 54, file) != 54) { //     54 ,   
		printf(" BMP-n");
		return false;
	}
	if (header[0] != 'B' || header[1] != 'M'){
		printf(" BMP-n");
		return 0;
	}
	dataPos = *(int*)&(header[0x0A]); //     
	imageSize = *(int*)&(header[0x22]); //    
	width = *(int*)&(header[0x12]); // 
	height = *(int*)&(header[0x16]); // 
	if (imageSize == 0)    imageSize = width*height * 3; //  *  * 3,  3 - 3   (RGB)
	if (dataPos == 0)      dataPos = 54; //   ,      
	data = new unsigned char[imageSize];
	fread(data, 1, imageSize, file);
	fclose(file);
	textureId;
	glGenTextures(1, &textureId);
	glBindTexture(GL_TEXTURE_2D, textureId);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, width, height, 0, GL_BGR, GL_UNSIGNED_BYTE, data);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	materials.texture = textureId;
	materials.ambient = vec4(0.2f, 0.2f, 0.2f, 1.0f);
	materials.diffuse = vec4(0.3f, 0.5f, 1.0f, 1.0f);
	materials.specular = vec4(0.8f, 0.8f, 0.8f, 1.0f);
	materials.emission = vec4(0.0f, 0.0f, 0.0f, 1.0f);
	materials.shininess = 20.0f;
	glBindTexture(GL_TEXTURE_2D, 0);
	free(data);
	return true;
}

void Object::Draw(GLuint program)
{	
	vec4 diff = vec4(2.0f, 2.0f, 2.0f, 1.0f);
	GLuint p = glGetUniformLocation(program, "material.diffuse");
	glUniform4fv(p,1,&materials.diffuse[0]);
	glEnableVertexAttribArray(vertexArray);
	if (textureFile != ""){
		glEnableVertexAttribArray(textureArray);
		glEnableVertexAttribArray(normalArray);
		glBindBuffer(GL_ARRAY_BUFFER, vbo);
		glVertexAttribPointer(vertexArray, 3, GL_FLOAT, GL_FALSE, sizeof(VertexTexture), 0);
		if (data.size() > 2){
			glVertexAttribPointer(textureArray, 2, GL_FLOAT, GL_FALSE, sizeof(VertexTexture), (const GLvoid*)12);
			glVertexAttribPointer(normalArray, 3, GL_FLOAT, GL_FALSE, sizeof(VertexTexture), (const GLvoid*)20);
		}
		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, ibo);
		if (data.size() > 2){
			glActiveTexture(GL_TEXTURE0);
			glBindTexture(GL_TEXTURE_2D, textureId);
		}
		glDrawElements(GL_TRIANGLES, vIndex.size(), GL_UNSIGNED_INT, 0);
		glDisableVertexAttribArray(normalArray);
		glDisableVertexAttribArray(textureArray);		
		glBindTexture(GL_TEXTURE_2D, 0);
	}
	else{
		glBindBuffer(GL_ARRAY_BUFFER, vbo);
		glVertexAttribPointer(vertexArray, 3, GL_FLOAT, GL_FALSE, sizeof(glm::vec3), 0);
		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, ibo);
		glDrawElements(GL_TRIANGLES, vIndex.size(), GL_UNSIGNED_INT, 0);
	}
	glDisableVertexAttribArray(vertexArray);
}

void Object::SetAttributeArrays(GLuint aa, int type)
{
	switch (type)
	{
	case 0:
		vertexArray = aa;
		break;
	case 1:
		colorArray = aa;
		break;
	case 2:
		textureArray = aa;
		break;
	case 3:
		normalArray = aa;
		break;
	}
}

void Object::LoadVertexes(vec3 v1)
{
	vertecies.push_back(v1);
	vertexCount = vertecies.size();
}

void Object::LoadIndexes(vec3 i1, int type)
{
	if (type == 0){
		vIndex.push_back(i1.x);
		vIndex.push_back(i1.y);
		vIndex.push_back(i1.z);
		indexCount = vIndex.size();
	}
	else if (type == 1){
		tIndex.push_back(i1.x);
		tIndex.push_back(i1.y);
		tIndex.push_back(i1.z);
	}
	else if (type == 2){
		nIndex.push_back(i1.x);
		nIndex.push_back(i1.y);
		nIndex.push_back(i1.z);
	}
	indexCount = vIndex.size();
}

void Object::LoadNormal(vec3 n1)
{
	normals.push_back(n1);
	normalsCount = normals.size();
}

void Object::LoadTextures(vec2 t1)
{
	texture.push_back(t1);
	textureCount = texture.size();
}

void Object::decIndexes()
{
	unsigned int offset = vIndex[0];
	for (auto i = 0; i < vIndex.size(); i++)
	{
		vIndex[i]-=offset;
	}
	offset = tIndex[0];
	for (auto i = 0; i < tIndex.size(); i++)
	{
		tIndex[i] -= offset;
	}
	offset = nIndex[0];
	for (auto i = 0; i < nIndex.size(); i++)
	{
		nIndex[i] -= offset;
	}
}