#include "UserInterface.h"
#include "resource.h"

UserInterface *UserInterface::Window = nullptr;

UserInterface::UserInterface()
{
	Window = this;
}


UserInterface::~UserInterface()
{
}

bool UserInterface::Initializate(HINSTANCE hInstance)
{
	hinst = hInstance;
	ZeroMemory(&wc,sizeof(WNDCLASSEX));
    wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_HREDRAW | CS_VREDRAW;
	wc.cbWndExtra = 0;
	wc.cbClsExtra = 0;
    wc.hbrBackground = (HBRUSH)COLOR_WINDOW;
	wc.hCursor = LoadCursor(0,IDC_ARROW);
	wc.hIcon = LoadIcon(0,IDI_APPLICATION);
	wc.hIconSm = LoadIcon(0,IDI_APPLICATION);
	wc.hInstance = hInstance;
	wc.lpfnWndProc = WndProc;
	wc.lpszClassName = "ENGINE";
	wc.lpszMenuName = 0;
	if(FAILED(RegisterClassEx(&wc)))
	{
		MessageBox(0,"Can't register class application.", "Error", MB_OK);
		return 0;	
	};
	return true;
}

bool UserInterface::CreateUserWindow(HINSTANCE hInstance, int width, int height)
{
	WNDCLASSEX wcd;
	ZeroMemory(&wcd, sizeof(WNDCLASSEX));
	wcd.hInstance = hInstance;
	wcd.lpfnWndProc = ChildProc;
	wcd.cbSize = sizeof(WNDCLASSEX);
	wcd.style = CS_HREDRAW | CS_VREDRAW;
	wcd.cbWndExtra = 0;
	wcd.cbClsExtra = 0;
	wcd.hbrBackground = (HBRUSH)COLOR_WINDOW;
	wcd.hIcon = LoadIcon(0, IDI_APPLICATION);
	wcd.hIconSm = LoadIcon(0, IDI_APPLICATION);
	wcd.lpszMenuName = 0;
	wcd.lpszClassName = "ChildWClass";
	wcd.hCursor = LoadCursor(NULL, IDC_CROSS);
	RegisterClassEx(&wcd);

	hwnd = CreateWindow("ENGINE", "Engine-Z", WS_OVERLAPPEDWINDOW, 0, 0, width, height, 0, 0, wc.hInstance, 0); 
	hwnd_c = CreateWindowEx(0, "ChildWClass", (LPCTSTR)NULL,
		WS_CHILD |  WS_MAXIMIZEBOX | WS_MINIMIZEBOX | WS_VISIBLE | WS_BORDER, 30, 50,
		600, 600, hwnd, (HMENU)ID_CHWINDOW, hInstance, NULL);
	bOpen = CreateWindow("BUTTON", "Open", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 700, 50, 50, 20, hwnd, (HMENU)ID_BUTTON_OPEN, hInstance, 0);
	labelTrack = CreateWindow("STATIC", "Offset", WS_CHILD | WS_VISIBLE , 700, 100, 100, 20, hwnd, (HMENU)ID_LABEL_TRACK, hInstance, 0);
	labelSize = CreateWindow("STATIC", "Size", WS_CHILD | WS_VISIBLE , 700, 170, 100, 20, hwnd, (HMENU)ID_LABEL_SIZE, hInstance, 0);
	hwndTrack = CreateTrackbar(hwnd, 0, 50, 0, 50,700,120);
	hwndSize = CreateTrackbar(hwnd, 0, 50, 0, 50,700, 190);
	HMENU menu = LoadMenu(hInstance,(LPCSTR)IDR_MENU1); 
	SetMenu(hwnd, menu);
	ShowWindow(hwnd, SW_SHOWMAXIMIZED);
	UpdateWindow(hwnd);
	return true;
}

void UserInterface::Run(Render *_render)
{
	render = _render;
	MSG msg;

	while(1)
	{
		if(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
			if(msg.message == WM_QUIT)
			break;
		}
		else
		{
			render->Run();
		}
	}
}

HWND WINAPI UserInterface::CreateTrackbar(HWND hwnd, UINT iMin, UINT iMax, UINT iSelMin, UINT iSelMax, UINT x, UINT y)
{
	InitCommonControls(); // loads common control's DLL 
	HWND trackHwnd = CreateWindowEx(
		0,                               // no extended styles 
		TRACKBAR_CLASS,                  // class name 
		"Trackbar Control",              // title (caption) 
		WS_CHILD |
		WS_VISIBLE |
		TBS_AUTOTICKS |
		TBS_ENABLESELRANGE,              // style 
		x, y,                          // position 
		200, 30,                         // size 
		hwnd,                         // parent window 
		(HMENU)ID_TRACKBAR,                     // control identifier 
		hinst,                         // instance 
		NULL                             // no WM_CREATE parameter 
		);
	SendMessage(trackHwnd, TBM_SETRANGE,(WPARAM)TRUE,(LPARAM)MAKELONG(iMin, iMax));
	SendMessage(trackHwnd, TBM_SETPAGESIZE,	0, (LPARAM)4); 
	SendMessage(trackHwnd, TBM_SETSEL,(WPARAM)FALSE,(LPARAM)MAKELONG(iSelMin, iSelMax));
	SendMessage(trackHwnd, TBM_SETPOS,(WPARAM)TRUE,(LPARAM)iSelMin);
	SetFocus(trackHwnd);
	return trackHwnd;
}

VOID WINAPI UserInterface::TBNotifications(HWND trackHwnd, WPARAM wParam, UINT iSelMin, UINT iSelMax)
{
	int dwPos;    // current position of slider 
	switch (LOWORD(wParam)) {
	case TB_ENDTRACK:
	{						
			if (trackHwnd == hwndTrack){
				dwPos = SendMessage(hwndTrack, TBM_GETPOS, 0, 0);
				render->rate = (float)(dwPos) / 1000.0f;
			}
			if (trackHwnd == hwndSize){
				dwPos = SendMessage(hwndSize, TBM_GETPOS, 0, 0);
				render->cutWidth = ((float)(dwPos) / 50.0f) + 0.1;
			}
	}
		break;
	default:
		break;
	}
}

LRESULT CALLBACK UserInterface::WndMessage(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg)
	{
	case WM_COMMAND:{
			WORD id = LOWORD(wParam);
			switch (id){
			case 3001:
				render->SetModelFile(OpenObjFile());
				break;
			case 3002:
				BOOL checked = IsDlgButtonChecked(hwnd, 3002);
				if (checked) {
					CheckDlgButton(hwnd, 3002, BST_UNCHECKED);
				}
				else {
					CheckDlgButton(hwnd, 3002, BST_CHECKED);
				}				
			}
		}
		break;
	case WM_HSCROLL:
		TBNotifications((HWND)lParam,wParam, 0, 50);
		break;
	case WM_KEYDOWN:
		switch (wParam)
		{
		case VK_LEFT:{
				Model* m = render->GetModelByName("object1");
				if(m!=nullptr)
					m->Translate(glm::vec3(2.1f, 0.0f, 0.0f));
				delete m;
				break;
			}
		}
		break;
	case WM_MOUSEWHEEL:{
				if (onChildWindow){
					int wheel = GET_WHEEL_DELTA_WPARAM(wParam);
					}						   
	}
	case WM_MOUSEMOVE:{
				RECT rect;
				GetWindowRect(hwnd_c, &rect);
				POINT pt = { LOWORD(lParam), HIWORD(lParam) };
				ClientToScreen(hwnd_c, &pt);
				if (!PtInRect(&rect, pt))
					onChildWindow = false;
				else onChildWindow = true;
		}
		break;
	case WM_DESTROY:
		PostQuitMessage(0);
		break;
	case WM_CREATE:
		break;
	}
	return DefWindowProc(hwnd,msg,wParam,lParam);
}

LRESULT CALLBACK UserInterface::ChildMessage(HWND hwnd_c, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg){
	case WM_CREATE:
		break;
	case WM_LBUTTONDOWN:
		GetCursorPos(&xy);
		break;
	case WM_MOUSEMOVE:{

			if (xy.x != 0 && xy.y != 0){
				POINT xy2;
				GetCursorPos(&xy2);
				float x = xy2.x - xy.x;
				float y = xy2.y - xy.y;
				float z = sqrtf(x*x + y*y);
				render->cam->Rotate(vec3(y,x,0.0f),0.05f);							 
				xy = xy2;
			}
		}
		break;
	case WM_LBUTTONUP:
		xy.x = 0;
		xy.y = 0;
		break;
	case WM_DESTROY: {
			PostQuitMessage(0);
			return 0;
		}
	}
	return DefWindowProc(hwnd_c, msg, wParam, lParam);
}

LRESULT CALLBACK WndProc(HWND hwnd,UINT msg,WPARAM wParam, LPARAM lParam){
	return UserInterface::GetWindow()->WndMessage(hwnd,msg,wParam,lParam);
}

LRESULT CALLBACK ChildProc(HWND hwnd_c, UINT msg, WPARAM wParam, LPARAM lParam){
	return UserInterface::GetWindow()->ChildMessage(hwnd_c, msg, wParam, lParam);
}

string UserInterface::OpenObjFile()
{
	char szFileName[MAX_PATH] = "";
	OPENFILENAME ofn;
	ZeroMemory(&ofn, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = hwnd;
	ofn.lpstrFilter = ".";
	ofn.lpstrFile = szFileName;
	ofn.nMaxFile = MAX_PATH;
	ofn.lpstrTitle = "Best test of all tests!";
	ofn.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;
	ofn.lpstrDefExt = "txt";	
	if (GetOpenFileName(&ofn)){
		char* fl = new char[100];
		strcpy(fl, ofn.lpstrFile);
		string result = fl;
		return result;
	}
	else return "";
}